@@ -29,7 +29,6 @@ class WP_Comments_List_Table extends WP_List_Table {
 	 * Constructor.
 	 *
 	 * @since 3.1.0
-	 * @access public
 	 *
 	 * @see WP_List_Table::__construct() for more information on default arguments.
 	 *
@@ -247,7 +246,11 @@ class WP_Comments_List_Table extends WP_List_Table {
 			$link = add_query_arg( 'comment_type', $comment_type, $link );
 
 		foreach ( $stati as $status => $label ) {
-			$class = ( $status === $comment_status ) ? ' class="current"' : '';
+			$current_link_attributes = '';
+
+			if ( $status === $comment_status ) {
+				$current_link_attributes = ' class="current" aria-current="page"';
+			}
 
 			if ( !isset( $num_comments->$status ) )
 				$num_comments->$status = 10;
@@ -259,7 +262,7 @@ class WP_Comments_List_Table extends WP_List_Table {
 			if ( !empty( $_REQUEST['s'] ) )
 				$link = add_query_arg( 's', esc_attr( wp_unslash( $_REQUEST['s'] ) ), $link );
 			*/
-			$status_links[ $status ] = "<a href='$link'$class>" . sprintf(
+			$status_links[ $status ] = "<a href='$link'$current_link_attributes>" . sprintf(
 				translate_nooped_plural( $label, $num_comments->$status ),
 				sprintf( '<span class="%s-count">%s</span>',
 					( 'moderated' === $status ) ? 'pending' : $status,
@@ -428,7 +431,6 @@ class WP_Comments_List_Table extends WP_List_Table {
 	 * Get the name of the default primary column.
 	 *
 	 * @since 4.3.0
-	 * @access protected
 	 *
 	 * @return string Name of the default primary column, in this case, 'comment'.
 	 */
@@ -437,7 +439,6 @@ class WP_Comments_List_Table extends WP_List_Table {
 	}
 
 	/**
-	 * @access public
 	 */
 	public function display() {
 		wp_nonce_field( "fetch-list-" . get_class( $this ), '_ajax_fetch_list_nonce' );
@@ -510,7 +511,6 @@ class WP_Comments_List_Table extends WP_List_Table {
  	 * Generate and display row actions links.
  	 *
  	 * @since 4.3.0
- 	 * @access protected
  	 *
  	 * @global string $comment_status Status for the current listed comments.
  	 *
@@ -715,7 +715,6 @@ class WP_Comments_List_Table extends WP_List_Table {
 	}
 
 	/**
-	 * @access public
 	 *
 	 * @param WP_Comment $comment The comment object.
 	 */
@@ -741,7 +740,6 @@ class WP_Comments_List_Table extends WP_List_Table {
 	}
 
 	/**
-	 * @access public
 	 *
 	 * @param WP_Comment $comment The comment object.
 	 */
