@@ -585,10 +585,16 @@
 
 		$message
 			.removeClass( 'updating-message' )
-			.addClass( 'updated-message installed button-disabled' )
+			.addClass( 'updated-message installed' )
 			.attr( 'aria-label', wp.updates.l10n.pluginInstalledLabel.replace( '%s', response.pluginName ) )
 			.text( wp.updates.l10n.pluginInstalled );
 
+		if ( $message.hasClass( 'button-primary' ) ) {
+			$message.addClass( 'button-primary-disabled' );
+		} else {
+			$message.addClass( 'button-disabled' );
+		}
+
 		wp.a11y.speak( wp.updates.l10n.installedMsg, 'polite' );
 
 		$document.trigger( 'wp-plugin-install-success', response );
@@ -597,10 +603,11 @@
 			setTimeout( function() {
 
 				// Transform the 'Install' button into an 'Activate' button.
-				$message.removeClass( 'install-now installed button-disabled updated-message' ).addClass( 'activate-now button-primary' )
+				$message.removeClass( 'install-now installed button-primary-disabled button-secondary-disabled button-disabled updated-message' )
+					.addClass( 'activate-now' )
 					.attr( 'href', response.activateUrl )
 					.attr( 'aria-label', wp.updates.l10n.activatePluginLabel.replace( '%s', response.pluginName ) )
-					.text( wp.updates.l10n.activatePlugin );
+					.text( response.activateLabel || wp.updates.l10n.activatePlugin );
 			}, 1000 );
 		}
 	};
@@ -2437,7 +2444,7 @@
 				return;
 			}
 
-			if ( 'undefined' === typeof message.action ) {
+			if ( ! message || 'undefined' === typeof message.action ) {
 				return;
 			}
 
