@@ -20,7 +20,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Taxonomy key.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $taxonomy;
@@ -29,7 +28,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Instance of a term meta fields object.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 * @var WP_REST_Term_Meta_Fields
 	 */
 	protected $meta;
@@ -38,7 +36,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Column to have the terms be sorted by.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $sort_column;
@@ -47,7 +44,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Number of terms that were found.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 * @var int
 	 */
 	protected $total_terms;
@@ -56,7 +52,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Constructor.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param string $taxonomy Taxonomy key.
 	 */
@@ -73,7 +68,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Registers the routes for the objects of the controller.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @see register_rest_route()
 	 */
@@ -136,7 +130,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks if a request has access to read terms in the specified taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return bool|WP_Error True if the request has read access, otherwise false or WP_Error object.
@@ -156,7 +149,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Retrieves terms associated with a taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
@@ -196,6 +188,16 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 			}
 		}
 
+		if ( isset( $prepared_args['orderby'] ) && isset( $request['orderby'] ) ) {
+			$orderby_mappings = array(
+				'include_slugs' => 'slug__in',
+			);
+
+			if ( isset( $orderby_mappings[ $request['orderby'] ] ) ) {
+				$prepared_args['orderby'] = $orderby_mappings[ $request['orderby'] ];
+			}
+		}
+
 		if ( isset( $registered['offset'] ) && ! empty( $request['offset'] ) ) {
 			$prepared_args['offset'] = $request['offset'];
 		} else {
@@ -324,7 +326,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks if a request has access to read or edit the specified term.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return bool|WP_Error True if the request has read access for the item, otherwise false or WP_Error object.
@@ -345,14 +346,12 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Gets a single term from a taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
 	 */
 	public function get_item( $request ) {
 		$term = $this->get_term( $request['id'] );
-
 		if ( is_wp_error( $term ) ) {
 			return $term;
 		}
@@ -366,7 +365,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks if a request has access to create a term.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return bool|WP_Error True if the request has access to create items, false or WP_Error object otherwise.
@@ -378,7 +376,10 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 		}
 
 		$taxonomy_obj = get_taxonomy( $this->taxonomy );
-		if ( ! current_user_can( $taxonomy_obj->cap->edit_terms ) ) {
+		if ( ( is_taxonomy_hierarchical( $this->taxonomy )
+				&& ! current_user_can( $taxonomy_obj->cap->edit_terms ) )
+			|| ( ! is_taxonomy_hierarchical( $this->taxonomy )
+				&& ! current_user_can( $taxonomy_obj->cap->assign_terms ) ) ) {
 			return new WP_Error( 'rest_cannot_create', __( 'Sorry, you are not allowed to create new terms.' ), array( 'status' => rest_authorization_required_code() ) );
 		}
 
@@ -389,7 +390,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Creates a single term in a taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
@@ -397,13 +397,13 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	public function create_item( $request ) {
 		if ( isset( $request['parent'] ) ) {
 			if ( ! is_taxonomy_hierarchical( $this->taxonomy ) ) {
-				return new WP_Error( 'rest_taxonomy_not_hierarchical', __( 'Can not set parent term, taxonomy is not hierarchical.' ), array( 'status' => 400 ) );
+				return new WP_Error( 'rest_taxonomy_not_hierarchical', __( 'Cannot set parent term, taxonomy is not hierarchical.' ), array( 'status' => 400 ) );
 			}
 
 			$parent = get_term( (int) $request['parent'], $this->taxonomy );
 
 			if ( ! $parent ) {
-				return new WP_Error( 'rest_term_invalid', __( "Parent term doesn't exist." ), array( 'status' => 400 ) );
+				return new WP_Error( 'rest_term_invalid', __( 'Parent term does not exist.' ), array( 'status' => 400 ) );
 			}
 		}
 
@@ -418,6 +418,7 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 			if ( $term_id = $term->get_error_data( 'term_exists' ) ) {
 				$existing_term = get_term( $term_id, $this->taxonomy );
 				$term->add_data( $existing_term->term_id, 'term_exists' );
+				$term->add_data( array( 'status' => 409, 'term_id' => $term_id ) );
 			}
 
 			return $term;
@@ -468,7 +469,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks if a request has access to update the specified term.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return bool|WP_Error True if the request has access to update the item, false or WP_Error object otherwise.
@@ -490,7 +490,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Updates a single term from a taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
@@ -503,13 +502,13 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 
 		if ( isset( $request['parent'] ) ) {
 			if ( ! is_taxonomy_hierarchical( $this->taxonomy ) ) {
-				return new WP_Error( 'rest_taxonomy_not_hierarchical', __( 'Can not set parent term, taxonomy is not hierarchical.' ), array( 'status' => 400 ) );
+				return new WP_Error( 'rest_taxonomy_not_hierarchical', __( 'Cannot set parent term, taxonomy is not hierarchical.' ), array( 'status' => 400 ) );
 			}
 
 			$parent = get_term( (int) $request['parent'], $this->taxonomy );
 
 			if ( ! $parent ) {
-				return new WP_Error( 'rest_term_invalid', __( "Parent term doesn't exist." ), array( 'status' => 400 ) );
+				return new WP_Error( 'rest_term_invalid', __( 'Parent term does not exist.' ), array( 'status' => 400 ) );
 			}
 		}
 
@@ -526,7 +525,7 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 
 		$term = get_term( $term->term_id, $this->taxonomy );
 
-		/* This action is documented in lib/endpoints/class-wp-rest-terms-controller.php */
+		/** This action is documented in wp-includes/rest-api/endpoints/class-wp-rest-terms-controller.php */
 		do_action( "rest_insert_{$this->taxonomy}", $term, $request, false );
 
 		$schema = $this->get_item_schema();
@@ -555,7 +554,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks if a request has access to delete the specified term.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return bool|WP_Error True if the request has access to delete the item, otherwise false or WP_Error object.
@@ -577,7 +575,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Deletes a single term from a taxonomy.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Full details about the request.
 	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
@@ -592,7 +589,8 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 
 		// We don't support trashing for terms.
 		if ( ! $force ) {
-			return new WP_Error( 'rest_trash_not_supported', __( 'Terms do not support trashing. Set force=true to delete.' ), array( 'status' => 501 ) );
+			/* translators: %s: force=true */
+			return new WP_Error( 'rest_trash_not_supported', sprintf( __( "Terms do not support trashing. Set '%s' to delete." ), 'force=true' ), array( 'status' => 501 ) );
 		}
 
 		$request->set_param( 'context', 'view' );
@@ -628,7 +626,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Prepares a single term for create or update.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param WP_REST_Request $request Request object.
 	 * @return object $prepared_term Term object.
@@ -681,7 +678,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Prepares a single term output for response.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @param obj             $item    Term object.
 	 * @param WP_REST_Request $request Request object.
@@ -689,42 +685,42 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 */
 	public function prepare_item_for_response( $item, $request ) {
 
-		$schema = $this->get_item_schema();
+		$fields = $this->get_fields_for_response( $request );
 		$data   = array();
 
-		if ( ! empty( $schema['properties']['id'] ) ) {
+		if ( in_array( 'id', $fields, true ) ) {
 			$data['id'] = (int) $item->term_id;
 		}
 
-		if ( ! empty( $schema['properties']['count'] ) ) {
+		if ( in_array( 'count', $fields, true ) ) {
 			$data['count'] = (int) $item->count;
 		}
 
-		if ( ! empty( $schema['properties']['description'] ) ) {
+		if ( in_array( 'description', $fields, true ) ) {
 			$data['description'] = $item->description;
 		}
 
-		if ( ! empty( $schema['properties']['link'] ) ) {
+		if ( in_array( 'link', $fields, true ) ) {
 			$data['link'] = get_term_link( $item );
 		}
 
-		if ( ! empty( $schema['properties']['name'] ) ) {
+		if ( in_array( 'name', $fields, true ) ) {
 			$data['name'] = $item->name;
 		}
 
-		if ( ! empty( $schema['properties']['slug'] ) ) {
+		if ( in_array( 'slug', $fields, true ) ) {
 			$data['slug'] = $item->slug;
 		}
 
-		if ( ! empty( $schema['properties']['taxonomy'] ) ) {
+		if ( in_array( 'taxonomy', $fields, true ) ) {
 			$data['taxonomy'] = $item->taxonomy;
 		}
 
-		if ( ! empty( $schema['properties']['parent'] ) ) {
+		if ( in_array( 'parent', $fields, true ) ) {
 			$data['parent'] = (int) $item->parent;
 		}
 
-		if ( ! empty( $schema['properties']['meta'] ) ) {
+		if ( in_array( 'meta', $fields, true ) ) {
 			$data['meta'] = $this->meta->get_value( $item->term_id, $request );
 		}
 
@@ -756,7 +752,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Prepares links for the request.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 *
 	 * @param object $term Term object.
 	 * @return array Links for the given term.
@@ -818,13 +813,12 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Retrieves the term's schema, conforming to JSON Schema.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @return array Item schema data.
 	 */
 	public function get_item_schema() {
 		$schema = array(
-			'$schema'    => 'http://json-schema.org/schema#',
+			'$schema'    => 'http://json-schema.org/draft-04/schema#',
 			'title'      => 'post_tag' === $this->taxonomy ? 'tag' : $this->taxonomy,
 			'type'       => 'object',
 			'properties' => array(
@@ -898,7 +892,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Retrieves the query params for collections.
 	 *
 	 * @since 4.7.0
-	 * @access public
 	 *
 	 * @return array Collection parameters.
 	 */
@@ -952,6 +945,7 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 				'include',
 				'name',
 				'slug',
+				'include_slugs',
 				'term_group',
 				'description',
 				'count',
@@ -978,8 +972,11 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 		);
 
 		$query_params['slug'] = array(
-			'description'       => __( 'Limit result set to terms with a specific slug.' ),
-			'type'              => 'string',
+			'description'       => __( 'Limit result set to terms with one or more specific slugs.' ),
+			'type'              => 'array',
+			'items'             => array(
+				'type'          => 'string'
+			),
 		);
 
 		/**
@@ -1004,7 +1001,6 @@ class WP_REST_Terms_Controller extends WP_REST_Controller {
 	 * Checks that the taxonomy is valid.
 	 *
 	 * @since 4.7.0
-	 * @access protected
 	 *
 	 * @param string $taxonomy Taxonomy to check.
 	 * @return bool Whether the taxonomy is allowed for REST management.
